/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Calendar code
 *
 * The Initial Developer of the Original Code is
 *   Markus Adrario <Mozilla@Adrario.de>
 * 
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*****************************
*   Initialization           *
*****************************/
var gMouseDown = "none";
var gMouseX = null;
var gMouseY = null;
var gHours = 0;
var gMinutes = 15;
var gHourCorrect = 0;
var gMode = 24; // sets 12 or 24 hour Mode - has yet to be implemented

/**
 * Initializes the Timepicker
 */
function onLoad() {
	window.addEventListener("keypress", onKeyPressed, true);
	
    // TODO: initialize gMode here
    // TODO: set the clock to the time passed in.
}

/*****************************
*   Event Handlers           *
*****************************/

/**
 * Handles the MouseDown event
 * when the left mouse button is held and is on one of the clock-hands,
 * sets the MouseDown status to "minute" or "hour"
 *
 * @param   aEvent        the mouseEvent triggered
 */
function onMouseDown(aEvent) {
    if (aEvent.button == 0) {
        gMouseX = aEvent.screenX;
        gMouseY = aEvent.screenY;
        
        // only start moving hand, if left mouse button is held and the hand is clicked on.
        if (aEvent.originalTarget.id == "minute-hand") {
            gMouseDown = "minute";
        }
        if (aEvent.originalTarget.id == "hour-hand") {
            gMouseDown = "hour";
        }
    }
    aEvent.stopPropagation();
}

/**
 * Handles the MouseUp event and resets the MouseDown status to "none".
 *
 * @param   aEvent           the mouseEvent triggered. 
 */
function onMouseUp(aEvent) {
    if (aEvent.button == 0) {
        gMouseDown = null;
    }
    aEvent.stopPropagation();
}

/**
 * Handles the MouseMove event.
 *
 * @param   aEvent           the mouseEvent triggered.
 */
function onMouseMove(aEvent) {
    if (!gMouseDown) {
        return;
    }
    let step = 1;
    if (gMouseDown == "minute" && aEvent.shiftKey == false) {
        step = 5;
    }
    setTime(getCoordinates(aEvent), gMouseDown, step);
    aEvent.stopPropagation();
}

function getAngle(aMode) {
  let angle = 0
  if (mode == "minute") {
    angle = 6 * gMinutes
  }
  else {
    angle = 30 * gHours
  }
  return angle
}

function updateTime(mode, step) {
  if (mode == "minute") {
    addSubtractTime("minute", step)
  }
  else {
    addSubtractTime("hour", step)
  }
}


// Modificar um pouco o cdigo e terminar esta parte.
function onKeyPressed(aEvent) {
  let keyCode = aEvent.keyCode;
  let mode = "minute";
  let accuracy = 0;
  switch(aEvent.keyCode) {
    case 38: // 38 up
      accuracy = 5;
      break;
    case 40: //40 down
      accuracy = -5;
      break;
    case 39:
      accuracy = 1;
      mode = "hour"
      break;
    case 37:
      accuracy = -1;
      mode = "hour"
      break;
  }
  if(mode == "minute" && aEvent.shiftKey == true)
    accuracy /= 5;
  updateTime(mode, accuracy)
}



function getHandPositionFromAngle(aAngle, aMode, aAccuracy) {
    let divider = 30 * aAccuracy;
    if (aMode == "minute") {
        divider = 6 * aAccuracy;
    }
    // To get a sector 15 degrees left & right of the numbers to snap to it,
    // we have to modify the angle for that calculation.
    aAngle -= divider / 2;
    let handPosition = Math.floor(aAngle / divider) + 1;
    
    if (handPosition < 0 || handPosition > 360 / divider - 1) {
        handPosition = 0;
    }
    handPosition *= aAccuracy;
    return handPosition;
}



/**
 * Handles the onClickevent
 * 
 * @param   aEvent           the mouseEvent triggered.
 */
function onClick(aEvent) {
    if (gMouseDown) {
        return;
    }

    let dx = Math.abs(aEvent.screenX - this.gMouseX);
    let dy = Math.abs(aEvent.screenY - this.gMouseY);
    // if the mouse was moved more than 2 pixels since mouseDown it's not a click.
    if ((dx*dx + dy*dy) > 4) {
        // theoretically this can cause unwanted behaviour, if somehone does a whole circle
        // with one clock-hand and ends the drag within the 2 pixels. nevertheless there is
        // quite a low chance for that.
        return;
    }
    let clickPoint = getCoordinates(aEvent);
    let circle = document.getElementById("hour-circle");

    // find out if the click was inside or outside the hour-circle.
    if (Math.sqrt(clickPoint[0] * clickPoint[0] + clickPoint[1] * clickPoint[1])
                  <= circle.r.baseVal.value) {
        setTime(clickPoint, "hour", 1);
    } else {
        setTime(clickPoint, "minute", 5);
    }
    aEvent.stopPropagation();
}

/**
 * Handles the DOMMouseScroll Event
 *
 * @param   aEvent          the MouseEvent triggered
 */
function scroll(aEvent) {
    let step = aEvent.detail > 0 ? -1 : 1;
    
    let hand = (aEvent.ctrlKey == true) ? "minute" : "hour";
    if (hand == "minute" && aEvent.shiftKey == false) {
            step *= 5;
    }
    addSubtractTime(hand, step);
    aEvent.preventDefault();
}

/**
 * handles the click on the +/- 12 h control
 */
function addTwelve(aEvent) {
    switchAmPm();
    gHours = gHourCorrect == 12 ? gHours + gHourCorrect : gHours - 12;

    updateTimeDisplay();
    aEvent.stopPropagation();
}

/*****************************
*   UI manipulation          *
*****************************/

/**
 *  sets the clock Hands to a new position based on the point at which the mouse-cursor is. 
 *
 * @param   aEvent           the mouseEvent triggered. 
 * @param   aHand            the clock-hand to be changed
 * @param   aAccuracy    in which steps the hand is moved.
 */
function setTime (aPoint, aHand, aAccuracy) {
    //let circle = document.getElementById("hour-circle");
    //let number = document.getElementById("hour1");
    let timeValue = getHandPositionFromAngle(calculateAngle(aPoint), aHand, aAccuracy);
    if (aHand == "hour") {
        if (timeValue == gHours - gHourCorrect) {
            return;
        }
        if (gHours - gHourCorrect == 11 && timeValue == 0) {
            switchAmPm();
        } else if (gHours % 12 == 0 && timeValue == 11) {
            switchAmPm();
        }
        rotateHand("hour", timeValue);
        gHours = timeValue + gHourCorrect;
    } else if (aHand == "minute") {
        if (timeValue == gMinutes) {
            return;
        }
        rotateHand("minute", timeValue);
        // only update hour hand here when dragging the minute-from
        if (gMouseDown) {
            if (gMinutes >= 55 && timeValue <= 5){
                addSubtractTime("hour", 1);
            } else if (gMinutes <= 5 && timeValue >= 55) {
                addSubtractTime("hour", -1);
            }
        }
        gMinutes = timeValue;
    } else {
        return;
    }
    updateTimeDisplay();

    return;
}

/**
 * Adds or subtracts a given Integer value from either minutes or hours.
 * 
 * @param aValue           The value to be added. (negative for subtraction)
 * @param aHand            The selected clock Hand to which the value is added
 */
function addSubtractTime(aHand, aValue) {
    let newValue = null;
    if (aHand == "hour") {
        newValue = (gHours - gHourCorrect) + aValue;
        if (newValue > 11) {
            switchAmPm();
            newValue -= 12;
        } else if (newValue < 0) {
            switchAmPm();
            newValue += 12;
        }
        gHours = newValue + gHourCorrect;
    } else if (aHand == "minute") {
        newValue = gMinutes + aValue;
        if (newValue > 59) {
            addSubtractTime("hour", 1);
            newValue -= 60;
        } else if (newValue < 0) {
            addSubtractTime("hour", -1);
            newValue += 60;
        }
        gMinutes = newValue;
    }
    rotateHand(aHand, newValue);
    updateTimeDisplay();
}

/**
 * Rotates a clock-hand to a given Position
 * 
 * @param aHand        The clock-hand zo be rotated
 * @param aValue       Hours or Minutes to be shown
 */
function rotateHand(aHand, aValue) {
    aValue *= (aHand == "hour") ? 30 : 6;

    let hand = document.getElementById(aHand + "-hand");
    hand.setAttribute("transform", "rotate(" + aValue + ",100,100)");
}

/**
 * Updates the Text display of the selected time.
 */
function updateTimeDisplay() {
    let timeText = document.getElementById("time-text");
    timeText.value = gHours + ":" + gMinutes;
}

/**
 * switches the Timepicker between morning and afternoon
 * and updates the hour values in the clock if 24 mode is active.
 */
function switchAmPm() {
    gHourCorrect = gHourCorrect == 0 ? 12 : 0;

    if (gMode == 24) {
        updateClockFace();
    }
}

/**
 * changes the clock face hour display from morning to 
 * afternoon in 24h mode.
 */
function updateClockFace() {
    for (i = 1; i < 12; i++) {
        text = document.getElementById("hour" + i);
        text.textContent = i + gHourCorrect;
    }
    // TODO: align numbers on clockface.
}

/*****************************
*   calculations             *
*****************************/

function getCoordinates(aEvent) {
    let box = document.getElementById("clock-face-box");
    
    // get the actual position relative to the center of the clock
    let relX = aEvent.clientX - (box.clientLeft + (box.clientWidth / 2));
    let relY = aEvent.clientY - (box.clientTop + (box.clientHeight / 2));
    
    let newPoint = new Array(relX, relY);
    return newPoint;
}

/**
 * calculates an angle relative to the upright from the coordinates
 * of the mouse pointer.
 *
 * @param  aEvent         The mouseEvent with the coordinates
 * @return angle          The calculated angle in degrees
 */
function calculateAngle(aPoint) {
    let box = document.getElementById("clock-face-box");
    
    // get the actual position relative to the center of the clock
    let X = aPoint[0];
    let Y = aPoint[1];
    
    // get the angle of the vector to the mouse position.
    // since we are working with relative positions to the center of the
    // clock, we can calculate the angle to the Y-Axis in a simplified formula.
    let cosine = (-Y) / (Math.sqrt(X*X + Y*Y));
    let angle = Math.round((180/Math.PI)*Math.acos(cosine));

    // since with this formula we only get unsigned angles up to 180,
    // we have to work around that for the other half.
    if (X < 0) {
        angle = 360 - angle;
    }
    return angle;
}

/**
 * calculates coordinates based on an angle and a distance from the center
 *
 * @param      aAngle            given angle
 * @param      aDistance         given distance
 * @return     [x,y]             the coordinates of the corresponding point
 */
function calculatePosition(aAngle, aDistance) {    
    // pretend the two coordinates are cathetes of a triangle and
    // calculate them accordingly.
    let X = Math.round(Math.sin(aAngle / (180/Math.PI)) * aDistance);
    let Y = -Math.round(Math.cos(aAngle / (180/Math.PI)) * aDistance);
    let point = new Array(X, Y);
    return point;
}

/**
 * calculates a time value (hour or minute) from a given angle
 *
 * @param   aAngle         the angle
 * @param   aMode          hour or minute, based on which hand is moved
 * @return  handPosition   the new position of the hand. ( hours or minutes)
 */
function getHandPositionFromAngle(aAngle, aMode, aAccuracy) {
    let divider = 30 * aAccuracy;
    if (aMode == "minute") {
        divider = 6 * aAccuracy;
    }
    // To get a sector 15 degrees left & right of the numbers to snap to it,
    // we have to modify the angle for that calculation.
    aAngle -= divider / 2;
    let handPosition = Math.floor(aAngle / divider) + 1;
    
    if (handPosition < 0 || handPosition > 360 / divider - 1) {
        handPosition = 0;
    }
    handPosition *= aAccuracy;
    return handPosition;
}

/**
 * makes sure the angle is between 0 and 360 degrees
 *
 * @param   aNumber           value to check
 * @return  angle             the verified angle
 */
function verifyAngle(aNumber) {
    if (aNumber < 0) {
        return 360 - aNumber;
    } else if (aNumber > 359) {
        return aNumber - 360;
    }
    return aNumber;
}